@@ -58,36 +58,68 @@ class Server:
         self._username = username
         self._password = password
 
-    async def async_get_players(self):
-        """Return Player for each device connected to LMS."""
+    async def async_get_players(self, search=None):
+        """
+        Return Player for each device connected to LMS.
+
+        Parameters:
+            search: filter the result by case-insensitive substring (optional)
+        """
         players = []
         data = await self.async_query("players", "status")
         if data is False:
             return None
         for player in data.get("players_loop", []):
-            players.append(Player(self, player["playerid"], player["name"]))
+            if search:
+                if search.lower() in player["name"].lower():
+                    players.append(Player(self, player["playerid"], player["name"]))
+            else:
+                players.append(Player(self, player["playerid"], player["name"]))
+        _LOGGER.debug("get_players(%s) returning players: %s", search, players)
         return players
 
     async def async_get_player(self, player_id=None, name=None):
-        """Return Player for a device, searching by name or player_id."""
+        """
+        Return Player for a device connected to server.
+
+        Parameters (one required):
+            player_id: The unique player_id reported by the server.
+            name: A substring for a case-insensitive match. Will return first of
+                  multiple matching results.
+        """
         if player_id:
             data = await self.async_query("status", player=player_id)
             if data:
-                name = data["player_name"]
-                if name:
-                    return Player(self, player_id, name)
-            _LOGGER.debug("Unable to find player with id: %s", player_id)
+                # an exact, case sensitive string match on the player name will
+                # also return a result. if that happened, search on name instead
+                # to retrieve accurate player_id
+                if player_id == data.get("player_name"):
+                    _LOGGER.info(
+                        "get_player(player_id=%s) called with player name.", player_id
+                    )
+                    return await self.async_get_player(name=player_id)
+                if "player_name" in data:
+                    return Player(self, player_id, data["player_name"])
+            _LOGGER.debug("Unable to find player with player_id: %s", player_id)
             return None
         if name:
-            players = await self.async_get_players()
-            for player in players:
-                if name.lower() == player.name.lower():
-                    return player
-            _LOGGER.debug("Unable to find player with name: %s", name)
+            players = await self.async_get_players(name)
+            if len(players) >= 1:
+                if len(players) > 1:
+                    _LOGGER.warning(
+                        "Found more than one player matching %s.", name,
+                    )
+                _LOGGER.debug("get_player(name=%s) return player %s.", name, players[0])
+                return players[0]
+            _LOGGER.debug("Unable to find player with name: %s.", name)
             return None
-        _LOGGER.error("get_player called without name or player_id")
+        _LOGGER.error("get_player() called without name or player_id.")
         return None
 
+    async def async_status(self):
+        """Return status of current server."""
+        return await self.async_query("serverstatus")
+
     async def async_query(self, *command, player=""):
         """Return result of query on the JSON-RPC connection."""
         auth = (
@@ -174,6 +206,20 @@ class Player:
         """Return the player ID, which is its MAC address."""
         return self._id
 
+    @property
+    def connected(self):
+        """
+        Return True if the player is connected to the LMS server.
+
+        The API call is less useful than it sounds, because after player has
+        been disconnected for a few minutes from the server, it disappears
+        altogether from the API. We still return False, not None, because it
+        still means the player is disconnected.
+        """
+        if "player_connected" in self._status:
+            return self._status["player_connected"] == 1
+        return False
+
     @property
     def power(self):
         """Return the power state of the device."""
